package com.Smartlook.Smartlook.flutter_smartlook

import com.smartlook.android.core.api.Session
import com.smartlook.android.core.api.Smartlook
import com.smartlook.android.core.api.User
import com.smartlook.android.core.bridge.BridgeInterface
import com.smartlook.android.core.bridge.model.BridgeFrameworkInfo
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.embedding.engine.plugins.FlutterPlugin.FlutterPluginBinding
import io.flutter.embedding.engine.plugins.activity.ActivityAware
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding
import io.flutter.plugin.common.BinaryMessenger
import io.flutter.plugin.common.EventChannel
import io.flutter.plugin.common.EventChannel.EventSink
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.PluginRegistry.Registrar
import java.net.URL

/**
 * SmartlookPlugin
 */
class  FlutterBridge: BridgeInterface
{
    override fun obtainFrameworkInfo(callback: (frameworkInfo: BridgeFrameworkInfo) -> Unit) {
        callback(BridgeFrameworkInfo(
            "Flutter",
            "4.0.6",
            frameworkVersion = "3.3.8",
          )
        )
    }
}

class SmartlookPlugin : FlutterPlugin {
    private var channel: MethodChannel? = null
    private var handler: MethodCallHandlerImpl? = null
    var smartlook:Smartlook = Smartlook.instance

    private fun setupChannel(messenger: BinaryMessenger) {

        channel = MethodChannel(messenger, METHOD_CHANNEL_NAME)
        handler = MethodCallHandlerImpl()
        channel!!.setMethodCallHandler(handler)
        val flutterInterfaceBridge = FlutterBridge()
        smartlook.bridgeInterfaces.add(flutterInterfaceBridge)

        EventChannel(messenger, EVENT_CHANNEL_NAME).setStreamHandler(
            object : EventChannel.StreamHandler {
                override fun onListen(arguments: Any?, eventSink: EventSink) {
                    smartlook.user.session.listeners += object : Session.Listener {
                        override fun onUrlChanged(url: URL) {
                            eventSink.success(url.path)
                        }
                    }
                    smartlook.user.listeners += object : User.Listener {
                        override fun onUrlChanged(url: URL) {
                            eventSink.success(url.path)
                        }
                    }
                }
                override fun onCancel(arguments: Any?) {
                    smartlook.user.listeners.clear()
                    smartlook.user.session.listeners.clear()
                }
            })
    }

    override fun onAttachedToEngine(binding: FlutterPluginBinding) {
        setupChannel(binding.binaryMessenger)
    }

    override fun onDetachedFromEngine(binding: FlutterPluginBinding) {
        channel!!.setMethodCallHandler(null)
        handler!!.teardown()
        handler = null
        channel = null
    }

    companion object {
        private const val METHOD_CHANNEL_NAME = "smartlook"
        private const val EVENT_CHANNEL_NAME = "smartlookEvent"
    }

}