import 'package:flutter_smartlook/flutter_smartlook.dart';
import 'package:flutter/material.dart';

class TestManager {
  Smartlook smartlook = Smartlook.instance;

  Future<void> testEventTracking() async {
    await smartlook.trackEvent('test_event_tracking');
    await smartlook.trackNavigationEnter('mock_screen');
    await smartlook.trackNavigationExit('mock_screen');
  }

  Future<void> testEventProperties() async {
    await smartlook.eventProperties.putString('test_key', value: 'test_value');
    final String? value = await smartlook.eventProperties.getString('test_key');
    assert(value == 'test_value');
    await smartlook.eventProperties.removeString('test_key');
    final String? removedValue =
        await smartlook.eventProperties.getString('test_key');
    assert(removedValue == null);
    await smartlook.eventProperties.putString('test_key', value: 'test_value');
    await smartlook.eventProperties.clear();
    final String? removedValueAfterClear =
        await smartlook.eventProperties.getString('test_key');
    assert(removedValueAfterClear == null);
  }

  Future<void> testUser() async {
    final String? userUrl = await smartlook.user.getUrl();
    assert(userUrl != null);
    await smartlook.user.setName('tester_name');
    await smartlook.user.setEmail('tester_email');
    await smartlook.user.properties.putString('test_key', value: 'test_value');
    final String? value = await smartlook.user.properties.getString('test_key');
    assert(value == 'test_value');
  }

  Future<void> testPreferences() async {
    ///already done
    //await smartlook.preferences.setRenderingMode(RenderingMode.native);
    //await smartlook.preferences.setWebViewEnabled(true)

    await smartlook.preferences.eventTracking.disableAll();
    await smartlook.preferences.eventTracking.enableAll();
    await smartlook.preferences.eventTracking.restoreDefault();
    await smartlook.preferences.eventTracking.interaction
        .setRageClickEnabled(true);
    await smartlook.preferences.eventTracking.interaction
        .setTrackUserInteraction(true);
  }

  Future<void> testState() async {
    final bool isAdaptiveFrameRateEnabled =
        await smartlook.state.getAdaptiveFrameRateEnabled();
    debugPrint("Frame rate enabled: $isAdaptiveFrameRateEnabled");
    final bool isSurfaceCaptureEnabled =
        await smartlook.state.getSurfaceCaptureEnabled();
    debugPrint("Surface capture enabled: $isSurfaceCaptureEnabled");
    final int usedFrameRate = await smartlook.state.getFrameRate();
    debugPrint("Used frame rate: $usedFrameRate");
    final String? usedProjectKey = await smartlook.state.getProjectKey();
    debugPrint("Project key: $usedProjectKey");
    final RenderingMode usedRenderingMode =
        await smartlook.state.getRenderingMode();
    debugPrint("Rendering mode: $usedRenderingMode");
    final bool isRecording =
        (await smartlook.state.getRecordingStatus()).isRecording;
    debugPrint("isRecording: $isRecording");
    final RecordingStatus usedRecordingStatus =
        await smartlook.state.getRecordingStatus();
    debugPrint("Recording status: $usedRecordingStatus");
  }

  Future<void> testUrlListener() async {
    final String? userUrl1 = await smartlook.user.getUrl();
    assert(userUrl1 != null);
    await smartlook.user.openNew();
    final String? userUrl2 = await smartlook.user.getUrl();
    assert(userUrl1 != userUrl2);
  }
}
