import 'package:flutter/material.dart';
import 'package:flutter_smartlook/flutter_smartlook.dart';
import 'package:flutter_smartlook_example/test_manager.dart';
import 'package:flutter_smartlook_example/timer_widget.dart';

class TestScreen extends StatefulWidget {
  const TestScreen({Key? key}) : super(key: key);

  @override
  State<TestScreen> createState() => _TestScreenState();
}

class _TestScreenState extends State<TestScreen> {
  final TestManager testManager = TestManager();
  Size containerSize = const Size(50, 50);
  GlobalKey key = GlobalKey(); // declare a global key

  @override
  void initState() {
    super.initState();
  }

  @override
  void dispose() {
    Smartlook.instance.setRecordingMask(null);
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(title: const Text("Test Screen")),
      body: Padding(
        padding: const EdgeInsets.all(40.0),
        child: Center(
          child: Column(
            mainAxisAlignment: MainAxisAlignment.spaceEvenly,
            children: [
              const TimerWidget(),
              ElevatedButton(
                onPressed: () async {
                  await testManager.testEventTracking();
                },
                child: const Text('Test event tracking'),
              ),
              ElevatedButton(
                onPressed: () async {
                  await testManager.testEventProperties();
                },
                child: const Text('Test event properties'),
              ),
              ElevatedButton(
                onPressed: () async {
                  await testManager.testUser();
                },
                child: const Text('Test user'),
              ),
              ElevatedButton(
                onPressed: () async {
                  await testManager.testPreferences();
                },
                child: const Text('Test preferences'),
              ),
              ElevatedButton(
                onPressed: () async {
                  await testManager.testState();
                },
                child: const Text('Test state'),
              ),
              ElevatedButton(
                onPressed: () async {
                  //TODO test what you need
                  Smartlook.instance.user.session.openNew();
                },
                child: const Text('Custom button'),
              ),
              Row(
                children: [
                  IconButton(
                      onPressed: () async {
                        await changeContainerSize(-10);
                        setState(() {});
                      },
                      icon: const Icon(Icons.remove)),
                  IconButton(
                      onPressed: () async {
                        await changeContainerSize(10);
                        setState(() {});
                      },
                      icon: const Icon(Icons.add)),
                ],
              ),
              Container(
                key: key,
                height: containerSize.height,
                width: containerSize.width,
                color: Colors.amber,
              ),
            ],
          ),
        ),
      ),
    );
  }

  Future<void> changeContainerSize(int changeVal) async {
    containerSize =
        Size(containerSize.width + changeVal, containerSize.height + changeVal);
    final RenderBox box = key.currentContext!.findRenderObject() as RenderBox;
    final Offset position =
        box.localToGlobal(Offset.zero); //this is global position
    final double x = position.dx;
    final double y = position.dy;

    await Smartlook.instance.setRecordingMask([
      RecordingMask(
        rect: Rect.fromLTWH(
            x, //* window.devicePixelRatio,
            y, //* window.devicePixelRatio,
            containerSize.width, //* window.devicePixelRatio,
            containerSize.height), //* window.devicePixelRatio),
        maskType: RecordingMaskType.covering,
      )
    ]);
  }
}
