import 'package:flutter_smartlook/src/enum.dart';
import 'package:flutter_smartlook/src/const_channels.dart';
import 'package:flutter_smartlook/src/preferences/event_tracking.dart';

/// Preferred configuration. The entered values represent
/// only the preferred configuration.
/// The resulting state may be different according to your account settings.
class SmartlookPreferences {
  const SmartlookPreferences();
  final EventTracking eventTracking = const EventTracking();

  /// Sets a [projectKey] of your project.
  Future<void> setProjectKey(String projectKey) async {
    await Channels.channel.invokeMethod('setProjectKey', {
      "projectKey": projectKey,
    });
  }

  /// Set preferred [frameRate] for a recording.
  Future<void> setFrameRate(int frameRate) async {
    await Channels.channel.invokeMethod('setFrameRate', {
      "frameRate": frameRate,
    });
  }

  /// Get preferred frame rate of a recording.
  Future<int?> getFrameRate() async {
    final int? frameRate =
        await Channels.channel.invokeMethod('getPreferencesFrameRate');
    return frameRate;
  }

  /// Set rendering mode: [RenderingMode.native], [RenderingMode.no_rendering].
  Future<void> setRenderingMode(RenderingMode renderingMode) async {
    await Channels.channel.invokeMethod('setRenderingMode', {
      "renderingMode": renderingMode.index,
    });
  }

  /// Whether to use Android Jobs instead of internal logic - android only.
  Future<void> setJobUploadEnabled(bool value) async {
    await Channels.channel.invokeMethod('setJobUploadEnabled', {
      "doUpload": value,
    });
  }

  /// Already set to false, should not be changed.
  Future<void> setAdaptiveFrameRateEnabled(bool value) async {
    await Channels.channel.invokeMethod('setAdaptiveFrameRateEnabled', {
      "adaptiveStatus": value,
    });
  }

  /// Already set to true, should not be changed.
  /// only for Android
  Future<void> setSurfaceCaptureEnabled(bool value) async {
    await Channels.channel.invokeMethod('setSurfaceCaptureEnabled', {
      "surfaceStatus": value,
    });
  }

  /// Enable web view, is hidden by default.
  Future<void> setWebViewEnabled(bool enabled) async {
    await Channels.channel.invokeMethod('setWebViewEnabled', {
      "enabled": enabled,
    });
  }
}
