import 'dart:async';

import 'package:flutter/material.dart';
import 'widget_tree/wireframe.dart';

/// Communication controller for wireframe
class SmartlookHelperController {
  SmartlookHelperController._privateConstructor();

  static final SmartlookHelperController instance =
      SmartlookHelperController._privateConstructor();

  final GlobalKey _containerKey = GlobalKey();

  Future<WireframeData> buildWireframeTree() async {
    final c = Completer<WireframeData>();

    final BuildContext ctx = _containerKey.currentContext!;

    final WireframeData wireframeData = WireframeData();
    await wireframeData.fillFromElement(ctx);
    c.complete(wireframeData);

    return c.future;
  }
}

/// Widget for tracking wireframe
class SmartlookHelperWidget extends StatelessWidget {
  const SmartlookHelperWidget({
    Key? key,
    required this.child,
  }) : super(key: key);

  final Widget child;

  @override
  Widget build(BuildContext context) {
    return RepaintBoundary(
      key: SmartlookHelperController.instance._containerKey,
      child: Container(child: child),
    );
  }
}

/// Wrap widget with sensitive data using [SensitiveDataWrapperWidget]
class SensitiveDataWrapperWidget extends StatelessWidget {
  final Widget child;
  final bool isSensitive;
  const SensitiveDataWrapperWidget({
    required this.child,
    this.isSensitive = true,
    Key? key,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return child;
  }
}
